/* this is to fix HTML->PDF rendering on Debian: wkhtmltopdf screws up the 
line heights otherwise */
table, table.th, table.td {
    line-height: 1.1em;
}

#main {
	padding: 4em;
}

/* Various headers */
#report-header {
	padding-bottom: 1em;
}

#patient-info {
	padding-bottom: 1em;
}

#variants {
	padding-top: 1em;
	padding-bottom: 1em;
}

#effects {
	margin-bottom: 1.5em;
}

#peptides {
	margin-bottom: 1.5em;
}

#args {
	padding-top: 2em;
	padding-bottom: 1em;
}

/* Patient info */
table.patient-info {
	width: 90%;
}

col.patient-info-column-one {
	width: 40%;
}

/* Command-line args */
table.args {
	width: 90%;
}

col.args-column-one {
	width: 40%;
}

/* Variants */
ol.main {
	padding-left: 1.2em;
	margin-left: 0.1em;
	font-size: 2em;
	color: #b3b3b3;
}

li.variant-list-item {
	border-left: 0.3em solid;
	padding: 0 2em 0 2em;
	margin-bottom: 2em;
	page-break-before: always;
	page-break-after: always;
}

li.variant-list-item:nth-child(odd) {
	border-left: 0.3em dotted;
	padding: 0 2em 0 2em;
	margin-bottom: 2em;
	page-break-before: always;
	page-break-after: always;
}

div.variant-span {
	font-size: 16px;
	color: black;
}

table.variant {
	margin-top: -2em;
	width: 90%;
}

col.variant-column-one {
	width: 40%;
}

col.variant-column-two {
	width: 60%;
}

thead.variant-head {
	font-size: 135%;
	margin-left: -2px;
}

td.variant-head {
	padding-left: 0.7em;
}

/* Peptides */
h4.peptides {
	page-break-before: always;
}

div.wt-epitopes {
	page-break-after: always;
	page-break-inside: avoid;
	margin: 2em 0 2em 2em;
}

table.wt-epitopes {
	font-size: 80%;
}

ol.peptides {
	padding-left: 0;
	list-style-position: inside;
}

li.peptide {
	page-break-inside: avoid;
	page-break-before: always;
}

table.peptide {
	margin: -1em 0 2em 2em;
	width: 80%;
}

span.mutant {
	background-color: yellow;
	border: 1px;
	border-style: solid;
	border-color: red;
	padding: 2px;
	margin-left: 2px;
	margin-right: 2px;
}

table.peptide-inner {
	width: 100%;
	border: none;
}

td.peptide-inner {
	padding: 0;
}

td.peptide-inner-header {
	font-weight: bold;
	text-align: center;
	font-size: 125%;
	padding: 0.75em;
}

col.peptide-data-column-one {
	width: 70%;
}

col.peptide-data-column-two {
	width: 30%;
}

col.epitope-data-column-one {
	width: 16%;
}

col.epitope-data-column-two {
	width: 16%;
}

col.epitope-data-column-three {
	width: 16%;
}

col.epitope-data-column-four {
	width: 16%;
}

col.epitope-data-column-five {
	width: 16%;
}

col.epitope-data-column-six {
	width: 16%;
}

table.epitope-inner {
	font-size: 80%;
	width: 100%;
	border: none;
}

/* Signatures */
table.signature {
	width: 70%;
}

table.signature-inner {
	width: 100%;
	border: none;
}

td.signature-inner {
	padding: 0;
}

col.signature-column-one {
	width: 30%;
}

col.signature-column-two {
	width: 70%;
}
